/***************************************************************************
                          satcalview.h  -  description
                             -------------------
    copyright            : (C) 2010 by Holtkamp
 Open Source software. See header of main.cpp for details.
 ***************************************************************************/

#ifndef SATCALVIEW_H
#define SATCALVIEW_H

// include files for QT
#include <QWidget>
#include<QScrollArea>
#include<QScrollBar>
#include <QPixmap>
#include <QPoint>
#include <QString>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QKeyEvent>

#include "satccom.h"

static const int GBL_MAPWIDTH = 721;   // width of map in pixel
static const int GBL_MAPHEIGHT = 361;  // heigth of map in pixel
static const int GBL_MAPTEXTHT = 140;  // height of text part of the map
static const int GBL_NSAVEPNT = 585;    // number of points of the map to be saved
static const int GBL_TEXTWIDTH = 860;
static const int GBL_TEXTLENGTH = 2500; // length of text buffer

class SatcalView : public QScrollArea
{
  Q_OBJECT

  public:
    SatcalView(QWidget *parent); 
    ~SatcalView();

    QString getTLEname();  // get the name of the current TLE file
    void setTLEname(const QString &tlename);  // set the name of the current TLE file
    QString getSaveName();  // get the name of the current Save file
    void setSaveName(const QString &tlename);  // set the name of the current Save file
    QString getTSaveName();  // get the name of the current TLE Save file
    void setTSaveName(const QString &tlename);  // set the name of the current TLE Save file
    void setFileName(char *fname);  // complete file name with path

    bool vUpdate(QString &shost, QString &spath, QString &sfnm);    
    void vFileSave();
    void vTLESave();
    void vPicture(const QImage &image);
    void vImage(int picsize);
    void vSelectSat();
    void vRemoveSat();
    int vNextSat();
    bool vInputDate();
    bool vInputLocation();
    void vSetIERS();
    void vSysConf();
    void vCalcAOS();
    void vAllSat();
    void vIridium();
    void vMatch();
    void vSwitch();
    void vEditOrb();
    void vDeltaV();
    void vLaunch();
    void vRendz();
    void vOrbDet();
    void vStep();
    void vMapDialog();
    void vRefresh();
    void vRefreshP();
    void vText();
    void vLocalPass();
    void vLocalInf();
    void vTmCheck(int timon);
    void vCurrent();
    void vPlotNext();
    void vPlotLast();
    void vForward();
    void vBackward();
    void vNames(bool on);
    void vPicSwitch(bool on);
    void vLeft(bool on);
    void vVisibility();
    void vMag();
    void vQuit();  // save configuration

    void astroEvent(int px, int py, bool lbutton);    
    void lsqfit (double* a, double* r);
    void getPlateConst();
    void equplt(double ra0,double dec0,double ra,double dec,
                double &xx,double &yy);
    void getRaDec(int px, int py, double &ra, double &dec);
    int getFsz();  // get font size
    bool getLeft();  // get left mouse button state
    void showtxt(bool tab);  // output text on screen
    void showSattxt(); // output list of satellite passes
    void showPassDetails(int y);
    void CmIridium();  // calculate Iridium Flares
    void getMapPos(int x, int y);
    void setFont(int fnt);
    void keyPress(int n);
    void scrollDsp(int dx, int dy);
    void goHome();  // moves to upper left corner
    void RT_Plot();  // RT plot
    bool PlotDelta();  // delta time plot
    void PlotWorld(QPainter &WorldMap);  // Plot the coordinate grid and continents
    void SatPlot(double t, int rngflg);  // plot satellite position
    void PicPlot(double t);  // plot picture
    void PltSunM(QPainter &WorldMap);   // sun or moon plot
    QPoint PxCoord (double lat, double lng); // get pixel coordinates
    QPoint PxCdSun (int sax, int occflg); // get sun map pixel coordinates
    QPoint PxSky (double elev, double azim); // get sky pixel coordinates
    void CoordPx (QPoint& p, double& lat, double& lng); // get coordinates from pixel
    void SkyPx (QPoint& p); // get sky pixel coord.
    void MapLand(QPainter &WorldMap); // draw continents and major islands
    void PltSky(QPainter &WorldMap); // draw skymap

    int origx;      // x pixel of coordinate origin
    int origy;      // y pixel of coordinate origin
    double xscale;  // scale in x (pixel / degrees)
    double yscale;  // scale in y (pixel / degrees)
    double dltend;  // MJD of end of delta plot in progress
    bool timsuspd;  // suspend timer actions if true

    int lastspx[GBL_MXSAT];   // last satellite x position
    int lastspy[GBL_MXSAT];   // last satellite y position

  protected:
    virtual void keyPressEvent(QKeyEvent *e);
    virtual void paintEvent( QPaintEvent *e );
    virtual void mousePressEvent( QMouseEvent *e );
    virtual void update();

  private:
    QPixmap *buffer;  // text output buffer
    QPixmap *buffM;   // map buffer (to store fixed map positions)
    QPixmap *buffPWM; // physical world map 
    QPixmap *buffP;   // plot buffer (for actual output)
    QPixmap *buffoot; // footer for the map
    QPixmap *buffPic; // buffer for the loaded picture
    QPixmap *buffTPic; // buffer for temporary (sized) picture
    QPixmap *buffPPic; // buffer for plotted picture

};

// functions for interchange between the global indata structure and dialogs
 void QsToChar(const QString &qname, char* cname, int len);  // copy QString to Char*
 void vsLastSat(QString &s, int &nsw, int &nrd, int &nsat);
 QString vsSatList();  // list of currently selected satellites
 void vsResetSatList();  // delete all selected satellites from sat list
 void vsSatSel();  // select satellite
 void vsFindSat(QString &s, int &nrd, int nrw);  // find sat
 void vsReComp(QString &s, int &nrd, int nrw, int morflg); // find sat from TLE
 QString vsSatName(int j);  // get name of j-th satellite
 void vsFileName(char *fname);  // complete file name with path
 int vsGetPassDTxt(double &t, QString &s, bool pole);  // detailed pass text
 void vsSavePassDt(int flsav );  // write pass details on save file
 void vsGetAdj(bool &usesdp4, bool &adjflg, double &adjtim, double &adjn0, double &adjra, int &wgs);
 void vsPutAdj(bool usesdp4, bool adjflg, double adjtim, double adjn0, double adjra, int wgs);
 void vsCkHyp(double& ecc, double& t0);  // check for hyperbolic numeric orbit
 void cvSGP4(double& xincl,double& xnodeo,double& eo,double& omegao,double& xmo,double& xno);
void vsGetPict(double &ra1, double &dec1, double &ra2, double &dec2);
void vsGetState(bool nmflg, double& f107, double& fb107, double& kp, int& tmje,
            double& pcr, double& pcd, double &pam, double &r0, double& r1, double& r2,
            double& v0, double& v1, double& v2, double& mjep);            
 void vsPutState(bool nmflg, int year, double day, double f107, double fb107, double kp,
            int tmje, double pcr, double pcd, double pam, double r0, double r1, double r2,
            double v0, double v1, double v2, bool cstate);
 void vsRefState(int tmjeold, int tmjenew, double t,
            double& r0, double& r1, double& r2, double& v0, double& v1, double& v2);
 
#endif
